# The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
from trytond.pool import PoolMeta, Pool
from trytond.pyson import Eval, Or, Id
from trytond.transaction import Transaction

__all__ = ['LoadOrder']


class LoadOrder:
    __name__ = 'carrier.load.order'
    __metaclass__ = PoolMeta

    @classmethod
    def __setup__(cls):
        super(LoadOrder, cls).__setup__()
        cls._transitions.add(('done', 'running'))
        cls._buttons.update({
            'run': {
                'icon': 'tryton-go-previous',
                'invisible': Or(Eval('state') != 'done', ~Eval('groups',
                    []).contains(Id('carrier_load_done2running',
                        'group_force_undo_load')
                ))
            },
        })
        cls._error_messages.update({
            'sale_state': 'Cannot cancel Sale "%s" due to it is confirmed.'})

    @classmethod
    def run(cls, records):
        pool = Pool()
        Sale = pool.get('sale.sale')
        Move = pool.get('stock.move')
        ShipmentOut = pool.get('stock.shipment.out')
        ShipmentInternal = pool.get('stock.shipment.internal')

        to_run = [r for r in records if r.state == 'done']
        if to_run:
            for item in to_run:
                if (item.sale and item.sale.origin == item and
                        item.sale.state not in ('draft', 'quotation')):
                    cls.raise_user_error('sale_state', item.sale.rec_name)

            sales = [r.sale for r in to_run if r.sale and r.sale.origin and
                r.sale.origin.__name__ == 'carrier.load.order']
            shipments_out = [r.shipment for r in to_run
                if r.type == 'out' and r.shipment]
            shipments_internal = [r.shipment for r in to_run
                if r.type == 'internal' and r.shipment]
            moves = [m for s in shipments_out + shipments_internal
                for m in s.moves]
            order_moves = [m for r in to_run for m in r.outgoing_moves]
            order_moves.extend([m for r in to_run for m in r.inventory_moves])
            with Transaction().set_context(check_origin=False,
                    check_shipment=False):
                Move.cancel(moves + order_moves)
                Move.delete(order_moves)
                if shipments_out:
                    ShipmentOut.delete(shipments_out)
                if shipments_internal:
                    ShipmentInternal.delete(shipments_internal)
            cls.write(to_run, {'shipment': None})
            if sales:
                Sale.draft(sales)
                Sale.delete(sales)

        super(LoadOrder, cls).run(records)
